package certi;

import certi.rti.impl.CertiRTIAmbassador;
import hla.rti.AttributeHandleSet;
import hla.rti.AttributeNotDefined;
import hla.rti.AttributeNotKnown;
import hla.rti.AttributeNotOwned;
import hla.rti.ConcurrentAccessAttempted;
import hla.rti.FederateInternalError;
import hla.rti.FederateNotExecutionMember;
import hla.rti.NameNotFound;
import hla.rti.ObjectAlreadyRegistered;
import hla.rti.ObjectClassNotDefined;
import hla.rti.ObjectClassNotPublished;
import hla.rti.ObjectNotKnown;
import hla.rti.OwnershipAcquisitionPending;
import hla.rti.RTIinternalError;
import hla.rti.RestoreInProgress;
import hla.rti.SaveInProgress;
import hla.rti.jlc.NullFederateAmbassador;
import hla.rti.jlc.RtiFactory;
import hla.rti.jlc.RtiFactoryFactory;
import hla.rti.FederateAmbassador;
import hla.rti.FederatesCurrentlyJoined;
import hla.rti.FederationExecutionAlreadyExists;
import hla.rti.FederationExecutionDoesNotExist;
import hla.rti.HandleIterator;
import hla.rti.ResignAction;
import hla.rti.RTIambassador;
import hla.rti.SuppliedAttributes;
import hla.rti.jlc.EncodingHelpers;
import java.io.File;

public class UavSend {

    private int myObject;
    private int textAttributeHandle;
    private int fomAttributeHandle;

    public void runFederate() throws Exception {
        System.out.println("     1. Get a link to the RTI");
        RtiFactory factory = RtiFactoryFactory.getRtiFactory("certi.rti.jlc.CertiRTIFactory");
        RTIambassador rtia = factory.createRtiAmbassador();

        System.out.println("     2. Create federation - nofail");
        try {
            File fom = new File("uav.fed");
            rtia.createFederationExecution("uav", fom.toURI().toURL());
        } catch (FederationExecutionAlreadyExists ex) {
            System.out.println("Federation already exists.");
        }

        System.out.println("     3. Join federation");
        FederateAmbassador mya = new MyFederateAmbassador();
        rtia.joinFederationExecution("uav-send", "uav", mya);

        System.out.println("     4. Initialize Federate Ambassador");
        ((MyFederateAmbassador) mya).initialize(rtia);


        System.out.println("     5 Initiate main loop");
        int i = 20;

        //HACK not keypressed :)
        while (i-- > 0) {
            System.out.println("     6 Loop");
            SuppliedAttributes attributes = RtiFactoryFactory.getRtiFactory("certi.rti.jlc.CertiRTIFactory").createSuppliedAttributes();

            byte[] textAttribute = EncodingHelpers.encodeString("text " + i);
            byte[] fomAttribute = EncodingHelpers.encodeFloat((float) Math.PI);

            attributes.add(textAttributeHandle, textAttribute);
            attributes.add(fomAttributeHandle, fomAttribute);

            byte[] tag = EncodingHelpers.encodeString("update");

            System.out.println("     6.1 Update Attributes");
            rtia.updateAttributeValues(myObject, attributes, tag);

            System.out.println("     6.2 Tick");
            ((CertiRTIAmbassador) rtia).tick(1.0, 1.0);

            Thread.sleep(1000);
        }

        System.out.println("     7 Resign federation execution");
        rtia.resignFederationExecution(ResignAction.DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES);

        System.out.println("     8 Destroy federation execution - nofail");
        try {
            rtia.destroyFederationExecution("uav");
        } catch (FederatesCurrentlyJoined ex) {
            System.out.println("Federates currently joined - can't destroy the execution.");
        } catch (FederationExecutionDoesNotExist ex) {
            System.out.println("Federation execution does not exists - can't destroy the execution.");
        }

        //TODO Add to interface or leave it as is?
        ((CertiRTIAmbassador) rtia).closeConnexion();
    }

    public static void main(String[] args) throws Exception {
        new UavSend().runFederate();
    }

    private class MyFederateAmbassador extends NullFederateAmbassador {

        public void initialize(RTIambassador rtia) throws NameNotFound, FederateNotExecutionMember, RTIinternalError, ObjectClassNotDefined, AttributeNotDefined, OwnershipAcquisitionPending, SaveInProgress, RestoreInProgress, ConcurrentAccessAttempted, ObjectClassNotPublished, ObjectAlreadyRegistered {
            System.out.println("     4.1 Get object class handle");
            int classHandle = rtia.getObjectClassHandle("SampleClass");

            System.out.println("     4.2 Get atribute handles");
            textAttributeHandle = rtia.getAttributeHandle("TextAttribute", classHandle);
            fomAttributeHandle = rtia.getAttributeHandle("FOMAttribute", classHandle);

            AttributeHandleSet attributes = RtiFactoryFactory.getRtiFactory("certi.rti.jlc.CertiRTIFactory").createAttributeHandleSet();
            attributes.add(textAttributeHandle);
            attributes.add(fomAttributeHandle);

            System.out.println("     4.3 Publish object");
            rtia.publishObjectClass(classHandle, attributes);

            System.out.println("     4.4 Register object instance");
            myObject = rtia.registerObjectInstance(classHandle, "HAF");
        }

        @Override
        public void startRegistrationForObjectClass(int theClass) throws ObjectClassNotPublished, FederateInternalError {
            super.startRegistrationForObjectClass(theClass);

            System.out.println(theClass);
        }

        @Override
        public void provideAttributeValueUpdate(int theObject, AttributeHandleSet theAttributes) throws ObjectNotKnown, AttributeNotKnown, AttributeNotOwned, FederateInternalError {
            super.provideAttributeValueUpdate(theObject, theAttributes);

            System.out.print(theObject);

            System.out.println("Attributes:");
            HandleIterator iterator = theAttributes.handles();
            for (int i = 1; i < theAttributes.size(); i++) {
                System.out.print(iterator.next());
            }
        }
    }
}
