// ----------------------------------------------------------------------------
// CERTI - HLA Run Time Infrastructure
// Copyright (C) 2007-2010 Andrej Pancik, Yannick Bisiaux, Ronan Bossard, Samuel Reese
//
// This program is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// ----------------------------------------------------------------------------
package certi.communication.messages;

import certi.communication.CertiException;
import certi.communication.CertiMessage;
import certi.communication.MessageBuffer;
import certi.communication.CertiMessageType;

/**
 *
 * @author apancik@gmail.com
 * @author <a href = "mailto:yannick.bisiaux@supaero.fr">Yannick Bisiaux</a>, <a href = "mailto:ronan.bossard@supaero.fr">Ronan Bossard</a>, <a href = "mailto:samuel.reese@supaero.fr">Samuel Reese</a>
 * @version 1.0
 *
 */
@SuppressWarnings("serial")
public class CreateFederationExecution extends CertiMessage {

    private String executionName;
    private String fed;

    /**
     * @return default Create_Federation_Execution message
     */
    public CreateFederationExecution() {
        super(CertiMessageType.CREATE_FEDERATION_EXECUTION);
        this.executionName = "";
        this.fed = "";
    }

    /**
     * @return Create_Federation_Execution message
     */
    public CreateFederationExecution(double federationTime, String executionName, String fed) {
        super(CertiMessageType.CREATE_FEDERATION_EXECUTION, federationTime);
        this.executionName = executionName;
        this.fed = fed;
    }

    /**
     * Writes the message to a buffer:
     * type of message, exception raised or not, federation time;
     * <code>federationName</code>, <code>fedId</code>.
     *
     * @param messageBuffer the <code>MessageBuffer</code>
     * to which the message is written.
     */
    @Override
    public void writeMessage(MessageBuffer messageBuffer) {
        super.writeMessage(messageBuffer); //header

        messageBuffer.write(this.executionName);
        messageBuffer.write(this.fed);
    }

    /**
     * Reads the message from a buffer:
     * type of message, exception raised or not, federation time;
     * <code>federationName</code>, <code>fedId</code>.
     *
     * @param messageBuffer the <code>MessageBuffer</code>
     * from which the message is read.
     */
    @Override
    public void readMessage(MessageBuffer messageBuffer) throws CertiException {
        super.readMessage(messageBuffer); //header
        
        this.executionName = messageBuffer.readString();
        this.fed = messageBuffer.readString();
    }

    @Override
    public String toString() {
        return (super.toString() + " " + this.executionName + " " + this.fed);
    }
}
