// ----------------------------------------------------------------------------
// CERTI - HLA Run Time Infrastructure
// Copyright (C) 2010 Andrej Pancik
//
// This program is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// ----------------------------------------------------------------------------
package certi.communication.messages;

import certi.communication.CertiException;
import certi.communication.CertiMessage;
import certi.communication.MessageBuffer;
import certi.communication.CertiMessageType;

public class GetObjectClassHandle extends CertiMessage {

    private long objectClass;
    private String name;
    private short attribute;

    public GetObjectClassHandle() {
        super(CertiMessageType.GET_OBJECT_CLASS_HANDLE);
        this.objectClass = 0;
        this.name = "";
        this.attribute = 0;
    }

    public GetObjectClassHandle(double federationTime, String theName) {
        super(CertiMessageType.GET_OBJECT_CLASS_HANDLE, federationTime);
        this.objectClass = 0; //TODO Verify
        this.name = theName;

    }

    @Override
    public void writeMessage(MessageBuffer messageBuffer) {
        super.writeMessage(messageBuffer);

        messageBuffer.write(objectClass);
        messageBuffer.write(name);
        messageBuffer.write(attribute);
    }

    @Override
    public void readMessage(MessageBuffer messageBuffer) throws CertiException {
        super.readMessage(messageBuffer);

        objectClass = messageBuffer.readLong();
        name = messageBuffer.readString();
        attribute = messageBuffer.readShort();
    }

    public long getObjectClass() {
        return objectClass;
    }

    @Override
    public String toString() {
        return (super.toString() + " " + this.objectClass + " " + this.name + " " + this.attribute);
    }
}
