// ----------------------------------------------------------------------------
// CERTI - HLA Run Time Infrastructure
// Copyright (C) 2010 Andrej Pancik
//
// This program is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// ----------------------------------------------------------------------------
package certi.communication.messages;

import certi.communication.CertiException;
import certi.communication.CertiMessage;
import certi.communication.MessageBuffer;
import certi.communication.CertiMessageType;

/**
 * @author apancik@gmail.com
 * @author <a href = "mailto:yannick.bisiaux@supaero.fr">Yannick Bisiaux</a>, <a href = "mailto:ronan.bossard@supaero.fr">Ronan Bossard</a>, <a href = "mailto:samuel.reese@supaero.fr">Samuel Reese</a>
 * @version 1.0
 *
 */
@SuppressWarnings("serial")
public class JoinFederationExecution extends CertiMessage {

    private short federate;
    private String federationName;
    private String federateName;

    /**
     * @return default Join_Federation_Execution message
     */
    public JoinFederationExecution() {
        super(CertiMessageType.JOIN_FEDERATION_EXECUTION);
        this.federate = (short) 0;
        this.federationName = "";
        this.federateName = "";
    }

    /**
     * @return Join_Federation_Execution message
     */
    public JoinFederationExecution(double federationTime, short federate, String federationName, String federateName) {
        super(CertiMessageType.JOIN_FEDERATION_EXECUTION, federationTime);
        this.federate = federate;
        this.federationName = federationName;
        this.federateName = federateName;
    }

    /**
     * Writes the message to a buffer:
     * type of message, exception raised or not, federation time;
     * <code>federate</code>, <code>federationName</code>, <code>federateName</code>.
     *
     * @param messageBuffer the <code>MessageBuffer</code>
     * to which the message is written.
     */
    @Override
    public void writeMessage(MessageBuffer messageBuffer) {
        //writeHeader
        super.writeMessage(messageBuffer);

        //writeBody
        messageBuffer.write(this.federate);
        messageBuffer.write(this.federationName);
        messageBuffer.write(this.federateName);
    }

    /**
     * Reads the message from a buffer:
     * type of message, exception raised or not, federation time;
     * <code>federate</code>, <code>federationName</code>, <code>federateName</code>.
     *
     * @param messageBuffer the <code>MessageBuffer</code>
     * from which the message is read.
     */
    @Override
    public void readMessage(MessageBuffer messageBuffer) throws CertiException {
        super.readMessage(messageBuffer); //header
        
            this.federate = messageBuffer.readShort();
            this.federationName = messageBuffer.readString();
            this.federateName = messageBuffer.readString();
        
    }

    public short getFederate() {
        return federate;
    }

    @Override
    public String toString() {
        return (super.toString() + " Federate:" + this.federate + ", " + this.federationName + ", " + this.federateName);
    }
}
