// ----------------------------------------------------------------------------
// CERTI - HLA Run Time Infrastructure
// Copyright (C) 2010 Andrej Pancik
//
// This program is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// ----------------------------------------------------------------------------
package certi.communication.messages;

import certi.communication.CertiException;
import certi.communication.CertiMessageType;
import certi.communication.CertiMessage;
import certi.communication.MessageBuffer;
import certi.rti.impl.CertiAttributeHandleSet;
import hla.rti.AttributeHandleSet;
import hla.rti.HandleIterator;

public class ProvideAttributeValueUpdate extends CertiMessage {

    private long objectClass;
    private AttributeHandleSet attributeHandleSet;

    public ProvideAttributeValueUpdate() {
        super(CertiMessageType.PROVIDE_ATTRIBUTE_VALUE_UPDATE);
    }

    public ProvideAttributeValueUpdate(double federationTime) {
        super(CertiMessageType.PROVIDE_ATTRIBUTE_VALUE_UPDATE, federationTime);
    }

    @Override
    public void writeMessage(MessageBuffer messageBuffer) {
        super.writeMessage(messageBuffer); //header

        messageBuffer.write(objectClass);
        messageBuffer.write((short) attributeHandleSet.size());

        HandleIterator iterator = attributeHandleSet.handles();
        messageBuffer.write((long) iterator.first()); //TODO PRETYPOVANIE
        for (int i = 1; i < attributeHandleSet.size(); i++) {
            messageBuffer.write((long) iterator.next()); //TODO PRETYPOVANIE
        }
    }

    @Override
    public void readMessage(MessageBuffer messageBuffer) throws CertiException {
        super.readMessage(messageBuffer); //header

        objectClass = messageBuffer.readLong();

        int size = messageBuffer.readShort();
        attributeHandleSet = new CertiAttributeHandleSet(size, messageBuffer);
    }

    public AttributeHandleSet getAttributeHandleSet() {
        return attributeHandleSet;
    }

    public long getObjectClass() {
        return objectClass;
    }

    @Override
    public String toString() {
        return (super.toString());
    }
}
