// ----------------------------------------------------------------------------
// CERTI - HLA Run Time Infrastructure
// Copyright (C) 2010 Andrej Pancik
//
// This program is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// ----------------------------------------------------------------------------
package certi.communication.messages;

import certi.communication.CertiException;
import certi.communication.CertiMessageType;
import certi.communication.CertiMessage;
import certi.communication.MessageBuffer;

public class StartRegistrationForObjectClass extends CertiMessage {

    private long objectClass;

    public StartRegistrationForObjectClass() {
        super(CertiMessageType.START_REGISTRATION_FOR_OBJECT_CLASS);
        objectClass = 0;
    }

    public long getObjectClass() {
        return objectClass;
    }

    public StartRegistrationForObjectClass(double federationTime, long objectClass) {
        super(CertiMessageType.START_REGISTRATION_FOR_OBJECT_CLASS, federationTime);
        this.objectClass = objectClass;
    }

    @Override
    public void writeMessage(MessageBuffer messageBuffer) {
        super.writeMessage(messageBuffer); //header

        messageBuffer.write(objectClass);
    }

    @Override
    public void readMessage(MessageBuffer messageBuffer) throws CertiException {
        super.readMessage(messageBuffer); //header

        objectClass = messageBuffer.readLong();
    }

    @Override
    public String toString() {
        return (super.toString());
    }
}
