// ----------------------------------------------------------------------------
// CERTI - HLA Run Time Infrastructure
// Copyright (C) 2010 Andrej Pancik
//
// This program is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// ----------------------------------------------------------------------------
package certi.communication.messages;

import certi.communication.CertiException;
import certi.communication.CertiMessage;
import certi.communication.MessageBuffer;
import certi.communication.CertiMessageType;
import certi.rti.impl.CertiSuppliedAttributes;
import hla.rti.ArrayIndexOutOfBounds;
import hla.rti.SuppliedAttributes;

public class UpdateAttributeValues extends CertiMessage {

    private long object;
    private long objectClass;
    private byte[] tag;
    private SuppliedAttributes suppliedAttributes;

    public UpdateAttributeValues() {
        super(CertiMessageType.UPDATE_ATTRIBUTE_VALUES);
        this.object = 0;
        this.objectClass = 0;
    }

    public UpdateAttributeValues(double federationTime, int theObject, SuppliedAttributes theAttributes, byte[] userSuppliedTag) {
        super(CertiMessageType.UPDATE_ATTRIBUTE_VALUES, federationTime);
        this.object = theObject;
        this.objectClass = 0;
        this.tag = userSuppliedTag;
        this.suppliedAttributes = theAttributes;
        //TODO email boolean v C++ ze naco je
    }

    @Override
    public void writeMessage(MessageBuffer messageBuffer) {
        super.writeMessage(messageBuffer);

        messageBuffer.write(objectClass);
        messageBuffer.write((short) suppliedAttributes.size());
        messageBuffer.write(object);

        messageBuffer.write(tag.length);
        messageBuffer.write(tag);


        try {
            for (int i = 0; i < suppliedAttributes.size(); i++) {
                messageBuffer.write((long) suppliedAttributes.getHandle(i));
            }

            for (int i = 0; i < suppliedAttributes.size(); i++) {
                messageBuffer.write(suppliedAttributes.getValue(i).length); //TODO check c++ code for reason of size-1-i
                messageBuffer.write(suppliedAttributes.getValue(i));
            }
            System.out.println();
        } catch (ArrayIndexOutOfBounds ex) {
            //TODO logit
            System.out.print("DEBUG: index array out of bounds");
        }

        //writeResignAction(messageBuffer);

        messageBuffer.write(false); //TODO what?
    }

    @Override
    public void readMessage(MessageBuffer messageBuffer) throws CertiException {
        super.readMessage(messageBuffer);

        objectClass = messageBuffer.readLong();
        int handleArraySize = messageBuffer.readShort();
        object = messageBuffer.readLong();

        tag = messageBuffer.readBytesWithSize();

        suppliedAttributes = new CertiSuppliedAttributes(handleArraySize, messageBuffer);
        // readResignAction(messageBuffer);
        messageBuffer.readBoolean();
    }

    @Override
    public String toString() {
        return (super.toString() + " " + this.objectClass + " " + this.object + " " + this.suppliedAttributes);
    }
}
