// ----------------------------------------------------------------------------
// CERTI - HLA Run Time Infrastructure
// Copyright (C) 2010 Andrej Pancik
//
// This program is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// ----------------------------------------------------------------------------
package certi.rti.impl;

import certi.communication.*;
import certi.communication.messages.*;
import hla.rti.*;
import hla.rti.jlc.RTIambassadorEx;
import hla.rti13.java1.MemoryExhausted;
import hla.rti13.java1.ValueCountExceeded;
import hla.rti13.java1.ValueLengthExceeded;
import java.io.IOException;
import java.io.InputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;

public class CertiRTIAmbassador implements RTIambassadorEx {

    private FederateAmbassador federateAmbassador;
    private Socket socket;
    private MessageBuffer messageBuffer = new MessageBuffer();
    private double federationTime = 0;

    public CertiRTIAmbassador() throws IOException {
        /*TODO rewrite - Very clumsy
         *  prefix + /bin/rtia getenv("CERTI_HOME") getenv("CERTI_RTIA")
        ServerSocket serverSocket = new ServerSocket(0);


        System.out.println("rtia -p " + serverSocket.getLocalPort());

        process = Runtime.getRuntime().exec("rtia -p " + serverSocket.getLocalPort());
        
        this.socket = serverSocket.accept(); */
        this.socket = new ServerSocket(2000).accept();
    }

    public boolean tick(double min, double max) throws RTIinternalError, ConcurrentAccessAttempted {
        try {
            return tickKernel(true, min, max);
        } catch (SpecifiedSaveLabelDoesNotExist ex) {
            //TODO log
            throw new RTIinternalError("SpecifiedSaveLabelDoesNotExist in tick");
        }
    }

    public boolean tick2() throws SpecifiedSaveLabelDoesNotExist, ConcurrentAccessAttempted, RTIinternalError {
        tickKernel(false, Double.MAX_VALUE, 0.0);
        return false;
    }

    private boolean tickKernel(boolean multiple, double minimum, double maximum) throws SpecifiedSaveLabelDoesNotExist, ConcurrentAccessAttempted, RTIinternalError {
        //TODO Code is hard to read - rewrite
        CertiMessage tickRequest = null;
        CertiMessage tickResponse = null;

        // Request callback(s) from the local RTIA
        tickRequest = new TickRequest(federationTime, multiple, minimum, maximum);

        tickRequest.writeMessage(this.messageBuffer);

        try {
            this.messageBuffer.send(this.socket.getOutputStream());
        } catch (IOException ex) {
            throw new RTIinternalError("NetworkError in tick() while sending TICK_REQUEST: " + ex.getMessage());
        }

        // Read response(s) from the local RTIA until CertiMessage::TICK_REQUEST is received.
        while (true) {
            try {
                InputStream in = this.socket.getInputStream();

                tickResponse = this.messageBuffer.receive(in);
                System.out.println("Received: " + tickResponse.toString() + "\n");
            } catch (IOException ex) {
                throw new RTIinternalError("NetworkError in tick() while receiving response: " + ex.getMessage());
            } catch (CertiException certiException) {
                // tick() may only throw exceptions defined in the HLA standard
                // the RTIA is responsible for sending 'allowed' exceptions only
                try {
                    translateException(certiException);
                } catch (SpecifiedSaveLabelDoesNotExist ex) {
                    throw ex;
                } catch (ConcurrentAccessAttempted ex) {
                    throw ex;
                } catch (RTIinternalError ex) {
                    throw ex;
                } catch (RuntimeException ex) {
                    throw ex;
                } catch (Exception ex) {//TODO LOG
                    System.out.print("Tick() error");
                }
            }

            // If the type is TICK_REQUEST, the tickKernel() has terminated.
            if (tickResponse.getType() == CertiMessageType.TICK_REQUEST) {
                return ((TickRequest) tickResponse).isMultiple();
            } else {
                try {
                    // Otherwise, the RTI calls a FederateAmbassador service.
                    callFederateAmbassador(tickResponse);
                } catch (RTIinternalError ex) {
                    // RTIA awaits TICK_REQUEST_NEXT, terminate the tick() processing
                    CertiMessage tickRequestStop = new TickRequestStop(federationTime);
                    try {
                        processRequest(tickRequestStop);
                    } catch (Exception tickResponseException) {
                        // ignore the response, ignore exceptions
                        // rep->type == CertiMessage::TICK_REQUEST;
                    }
                    // ignore the response and re-throw the original exception
                    throw ex;
                }

                CertiMessage tickNext = new TickRequestNext(federationTime);

                tickNext.writeMessage(this.messageBuffer);

                try {
                    this.messageBuffer.send(this.socket.getOutputStream());
                } catch (IOException ex) {
                    throw new RTIinternalError("NetworkError in tick() while sending TICK_REQUEST_NEXT: " + ex.getMessage());
                }
            }
        }
    }

    public void closeConnexion() {
        //TODO look into http://java.sun.com/j2se/1.4.2/docs/api/java/lang/Runtime.html#addShutdownHook%28java.lang.Thread%29

        CertiMessage request = new CloseConnexion(federationTime);
        try {
            System.out.print("CLOSING CONNECTION");
            CertiMessage response = processRequest(request);
        } catch (Exception e) {
            //TODO log
            System.out.print("CLOSING CONNECTION error");
        }
    }

    public void createFederationExecution(String executionName, URL fed) throws FederationExecutionAlreadyExists, CouldNotOpenFED, ErrorReadingFED, RTIinternalError, ConcurrentAccessAttempted {
        if (executionName == null || executionName.length() == 0) {
            throw new RTIinternalError("Incorrect or empty execution name");
        }
        if (fed == null) {
            throw new RTIinternalError("Incorrect or empty fed file name");
        }
        //HACK nebude fungovat na unixe hlavne ta cast substring
        CertiMessage request = new CreateFederationExecution(federationTime, executionName, fed.getFile().substring(1));
        try {
            CertiMessage response = processRequest(request);
        } catch (FederationExecutionAlreadyExists ex) {
            throw ex;
        } catch (CouldNotOpenFED ex) {
            throw ex;
        } catch (ErrorReadingFED ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (ConcurrentAccessAttempted ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {
            //TODO log
            System.out.println("debug:" + ex.toString());
        }
    }

    public void destroyFederationExecution(String executionName) throws FederatesCurrentlyJoined, FederationExecutionDoesNotExist, RTIinternalError, ConcurrentAccessAttempted {
        CertiMessage request = new DestroyFederationExecution(federationTime, executionName);
        try {
            CertiMessage response = processRequest(request);
        } catch (FederatesCurrentlyJoined ex) {
            throw ex;
        } catch (FederationExecutionDoesNotExist ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (ConcurrentAccessAttempted ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {
            //TODO Log
        }
    }

    public int joinFederationExecution(String federateName, String federationExecutionName, FederateAmbassador federateReference) throws FederateAlreadyExecutionMember, FederationExecutionDoesNotExist, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        if (federateName == null || federateName.length() == 0) {
            throw new RTIinternalError("Incorrect or empty federate name");
        }
        if (federationExecutionName == null || federationExecutionName.length() == 0) {
            throw new RTIinternalError("Incorrect or empty federation name");
        }
        if (federateReference == null) {
            throw new RTIinternalError("Federate reference is null");
        }
        federateAmbassador = federateReference;

        //TODO what if pointer is greater than short?
        CertiMessage request = new JoinFederationExecution(federationTime, (short) 0, federationExecutionName, federateName);
        try {
            CertiMessage response = processRequest(request);
            return ((JoinFederationExecution) request).getFederate();
        } catch (FederateAlreadyExecutionMember ex) {
            throw ex;
        } catch (FederationExecutionDoesNotExist ex) {
            throw ex;
        } catch (SaveInProgress ex) {
            throw ex;
        } catch (RestoreInProgress ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (ConcurrentAccessAttempted ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {
            //TODO Log it
            System.out.println(ex.toString());
        }

        return -1;
    }

    public int joinFederationExecution(String federateType, String federationExecutionName, FederateAmbassador federateReference, MobileFederateServices serviceReferences) throws FederateAlreadyExecutionMember, FederationExecutionDoesNotExist, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void resignFederationExecution(int resignAction) throws FederateOwnsAttributes, FederateNotExecutionMember, InvalidResignAction, RTIinternalError, ConcurrentAccessAttempted {
        CertiMessage request = new ResignFederationExecution(federationTime, resignAction);
        try {
            CertiMessage response = processRequest(request);
        } catch (FederateOwnsAttributes ex) {
            throw ex;
        } catch (FederateNotExecutionMember ex) {
            throw ex;
        } catch (InvalidResignAction ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (ConcurrentAccessAttempted ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {//TODO log it
            System.out.println(ex.toString());
        }
    }

    public void registerFederationSynchronizationPoint(String synchronizationPointLabel, byte[] userSuppliedTag) throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void registerFederationSynchronizationPoint(String synchronizationPointLabel, byte[] userSuppliedTag, FederateHandleSet synchronizationSet) throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void synchronizationPointAchieved(String synchronizationPointLabel) throws SynchronizationLabelNotAnnounced, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void requestFederationSave(String label, LogicalTime theTime) throws FederationTimeAlreadyPassed, InvalidFederationTime, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void requestFederationSave(String label) throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void federateSaveBegun() throws SaveNotInitiated, FederateNotExecutionMember, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void federateSaveComplete() throws SaveNotInitiated, FederateNotExecutionMember, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void federateSaveNotComplete() throws SaveNotInitiated, FederateNotExecutionMember, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void requestFederationRestore(String label) throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void federateRestoreComplete() throws RestoreNotRequested, FederateNotExecutionMember, SaveInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void federateRestoreNotComplete() throws RestoreNotRequested, FederateNotExecutionMember, SaveInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void publishObjectClass(int theClass, AttributeHandleSet attributeList) throws ObjectClassNotDefined, AttributeNotDefined, OwnershipAcquisitionPending, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {

        CertiMessage request = new PublishObjectClass(federationTime, theClass, attributeList);

        try {
            processRequest(request);
        } catch (ObjectClassNotDefined ex) {
            throw ex;
        } catch (AttributeNotDefined ex) {
            throw ex;
        } //catch( OwnershipAcquisitionPending ex) {throw ex;}
        catch (FederateNotExecutionMember ex) {
            throw ex;
        } catch (SaveInProgress ex) {
            throw ex;
        } catch (RestoreInProgress ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (ConcurrentAccessAttempted ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {//TODO Log
        }
    }

    public void unpublishObjectClass(int theClass) throws ObjectClassNotDefined, ObjectClassNotPublished, OwnershipAcquisitionPending, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void publishInteractionClass(int theInteraction) throws InteractionClassNotDefined, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void unpublishInteractionClass(int theInteraction) throws InteractionClassNotDefined, InteractionClassNotPublished, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void subscribeObjectClassAttributes(int theClass, AttributeHandleSet attributeList) throws ObjectClassNotDefined, AttributeNotDefined, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void subscribeObjectClassAttributesPassively(int theClass, AttributeHandleSet attributeList) throws ObjectClassNotDefined, AttributeNotDefined, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void unsubscribeObjectClass(int theClass) throws ObjectClassNotDefined, ObjectClassNotSubscribed, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void subscribeInteractionClass(int theClass) throws InteractionClassNotDefined, FederateNotExecutionMember, FederateLoggingServiceCalls, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void subscribeInteractionClassPassively(int theClass) throws InteractionClassNotDefined, FederateNotExecutionMember, FederateLoggingServiceCalls, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void unsubscribeInteractionClass(int theClass) throws InteractionClassNotDefined, InteractionClassNotSubscribed, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int registerObjectInstance(int theClass) throws ObjectClassNotDefined, ObjectClassNotPublished, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int registerObjectInstance(int theClass, String theObject) throws ObjectClassNotDefined, ObjectClassNotPublished, ObjectAlreadyRegistered, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        if (theObject == null || theObject.length() == 0) {
            throw new RTIinternalError("Incorrect or empty object name");
        }

        CertiMessage request = new RegisterObjectInstance(federationTime, theClass, theObject);

        try {
            CertiMessage response = processRequest(request);
            return (int) ((RegisterObjectInstance) response).getObject();
        } catch (ObjectClassNotDefined ex) {
            throw ex;
        } catch (ObjectClassNotPublished ex) {
            throw ex;
        } catch (ObjectAlreadyRegistered ex) {
            throw ex;
        } catch (FederateNotExecutionMember ex) {
            throw ex;
        } catch (SaveInProgress ex) {
            throw ex;
        } catch (RestoreInProgress ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (ConcurrentAccessAttempted ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {
            //TODO Log it
            System.out.println("LOG " + ex.toString());
        }

        return -1;
    }

    public void updateAttributeValues(int theObject, SuppliedAttributes theAttributes, byte[] userSuppliedTag) throws ObjectNotKnown, AttributeNotDefined, AttributeNotOwned, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        if (userSuppliedTag == null) {
            throw new RTIinternalError("Calling updateAttributeValues with Tag NULL");
        }

        CertiMessage request = new UpdateAttributeValues(federationTime, theObject, theAttributes, userSuppliedTag);

        try {
            CertiMessage response = processRequest(request);
        } catch (ObjectNotKnown ex) {
            throw ex;
        } catch (AttributeNotDefined ex) {
            throw ex;
        } catch (AttributeNotOwned ex) {
            throw ex;
        } catch (FederateNotExecutionMember ex) {
            throw ex;
        } catch (SaveInProgress ex) {
            throw ex;
        } catch (RestoreInProgress ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (ConcurrentAccessAttempted ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {
            //TODO Log
        }
    }

    public EventRetractionHandle updateAttributeValues(int theObject, SuppliedAttributes theAttributes, byte[] userSuppliedTag, LogicalTime theTime) throws ObjectNotKnown, AttributeNotDefined, AttributeNotOwned, InvalidFederationTime, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void sendInteraction(int theInteraction, SuppliedParameters theParameters, byte[] userSuppliedTag) throws InteractionClassNotDefined, InteractionClassNotPublished, InteractionParameterNotDefined, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public EventRetractionHandle sendInteraction(int theInteraction, SuppliedParameters theParameters, byte[] userSuppliedTag, LogicalTime theTime) throws InteractionClassNotDefined, InteractionClassNotPublished, InteractionParameterNotDefined, InvalidFederationTime, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void deleteObjectInstance(int ObjectHandle, byte[] userSuppliedTag) throws ObjectNotKnown, DeletePrivilegeNotHeld, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public EventRetractionHandle deleteObjectInstance(int ObjectHandle, byte[] userSuppliedTag, LogicalTime theTime) throws ObjectNotKnown, DeletePrivilegeNotHeld, InvalidFederationTime, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void localDeleteObjectInstance(int ObjectHandle) throws ObjectNotKnown, FederateOwnsAttributes, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void changeAttributeTransportationType(int theObject, AttributeHandleSet theAttributes, int theType) throws ObjectNotKnown, AttributeNotDefined, AttributeNotOwned, InvalidTransportationHandle, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void changeInteractionTransportationType(int theClass, int theType) throws InteractionClassNotDefined, InteractionClassNotPublished, InvalidTransportationHandle, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void requestObjectAttributeValueUpdate(int theObject, AttributeHandleSet theAttributes) throws ObjectNotKnown, AttributeNotDefined, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void requestClassAttributeValueUpdate(int theClass, AttributeHandleSet theAttributes) throws ObjectClassNotDefined, AttributeNotDefined, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void unconditionalAttributeOwnershipDivestiture(int theObject, AttributeHandleSet theAttributes) throws ObjectNotKnown, AttributeNotDefined, AttributeNotOwned, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void negotiatedAttributeOwnershipDivestiture(int theObject, AttributeHandleSet theAttributes, byte[] userSuppliedTag) throws ObjectNotKnown, AttributeNotDefined, AttributeNotOwned, AttributeAlreadyBeingDivested, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void attributeOwnershipAcquisition(int theObject, AttributeHandleSet desiredAttributes, byte[] userSuppliedTag) throws ObjectNotKnown, ObjectClassNotPublished, AttributeNotDefined, AttributeNotPublished, FederateOwnsAttributes, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void attributeOwnershipAcquisitionIfAvailable(int theObject, AttributeHandleSet desiredAttributes) throws ObjectNotKnown, ObjectClassNotPublished, AttributeNotDefined, AttributeNotPublished, FederateOwnsAttributes, AttributeAlreadyBeingAcquired, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public AttributeHandleSet attributeOwnershipReleaseResponse(int theObject, AttributeHandleSet theAttributes) throws ObjectNotKnown, AttributeNotDefined, AttributeNotOwned, FederateWasNotAskedToReleaseAttribute, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void cancelNegotiatedAttributeOwnershipDivestiture(int theObject, AttributeHandleSet theAttributes) throws ObjectNotKnown, AttributeNotDefined, AttributeNotOwned, AttributeDivestitureWasNotRequested, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void cancelAttributeOwnershipAcquisition(int theObject, AttributeHandleSet theAttributes) throws ObjectNotKnown, AttributeNotDefined, AttributeAlreadyOwned, AttributeAcquisitionWasNotRequested, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void queryAttributeOwnership(int theObject, int theAttribute) throws ObjectNotKnown, AttributeNotDefined, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public boolean isAttributeOwnedByFederate(int theObject, int theAttribute) throws ObjectNotKnown, AttributeNotDefined, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void enableTimeRegulation(LogicalTime theFederateTime, LogicalTimeInterval theLookahead) throws TimeRegulationAlreadyEnabled, EnableTimeRegulationPending, TimeAdvanceAlreadyInProgress, InvalidFederationTime, InvalidLookahead, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void disableTimeRegulation() throws TimeRegulationWasNotEnabled, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void enableTimeConstrained() throws TimeConstrainedAlreadyEnabled, EnableTimeConstrainedPending, TimeAdvanceAlreadyInProgress, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void disableTimeConstrained() throws TimeConstrainedWasNotEnabled, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void timeAdvanceRequest(LogicalTime theTime) throws InvalidFederationTime, FederationTimeAlreadyPassed, TimeAdvanceAlreadyInProgress, EnableTimeRegulationPending, EnableTimeConstrainedPending, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void timeAdvanceRequestAvailable(LogicalTime theTime) throws InvalidFederationTime, FederationTimeAlreadyPassed, TimeAdvanceAlreadyInProgress, EnableTimeRegulationPending, EnableTimeConstrainedPending, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void nextEventRequest(LogicalTime theTime) throws InvalidFederationTime, FederationTimeAlreadyPassed, TimeAdvanceAlreadyInProgress, EnableTimeRegulationPending, EnableTimeConstrainedPending, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void nextEventRequestAvailable(LogicalTime theTime) throws InvalidFederationTime, FederationTimeAlreadyPassed, TimeAdvanceAlreadyInProgress, EnableTimeRegulationPending, EnableTimeConstrainedPending, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void flushQueueRequest(LogicalTime theTime) throws InvalidFederationTime, FederationTimeAlreadyPassed, TimeAdvanceAlreadyInProgress, EnableTimeRegulationPending, EnableTimeConstrainedPending, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void enableAsynchronousDelivery() throws AsynchronousDeliveryAlreadyEnabled, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void disableAsynchronousDelivery() throws AsynchronousDeliveryAlreadyDisabled, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public LogicalTime queryLBTS() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public LogicalTime queryFederateTime() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public LogicalTime queryMinNextEventTime() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void modifyLookahead(LogicalTimeInterval theLookahead) throws InvalidLookahead, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public LogicalTimeInterval queryLookahead() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void retract(EventRetractionHandle theHandle) throws InvalidRetractionHandle, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void changeAttributeOrderType(int theObject, AttributeHandleSet theAttributes, int theType) throws ObjectNotKnown, AttributeNotDefined, AttributeNotOwned, InvalidOrderingHandle, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void changeInteractionOrderType(int theClass, int theType) throws InteractionClassNotDefined, InteractionClassNotPublished, InvalidOrderingHandle, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Region createRegion(int spaceHandle, int numberOfExtents) throws SpaceNotDefined, InvalidExtents, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void notifyOfRegionModification(Region modifiedRegionInstance) throws RegionNotKnown, InvalidExtents, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void deleteRegion(Region theRegion) throws RegionNotKnown, RegionInUse, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int registerObjectInstanceWithRegion(int theClass, int[] theAttributes, Region[] theRegions) throws ObjectClassNotDefined, ObjectClassNotPublished, AttributeNotDefined, AttributeNotPublished, RegionNotKnown, InvalidRegionContext, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int registerObjectInstanceWithRegion(int theClass, String theObject, int[] theAttributes, Region[] theRegions) throws ObjectClassNotDefined, ObjectClassNotPublished, AttributeNotDefined, AttributeNotPublished, RegionNotKnown, InvalidRegionContext, ObjectAlreadyRegistered, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void associateRegionForUpdates(Region theRegion, int theObject, AttributeHandleSet theAttributes) throws ObjectNotKnown, AttributeNotDefined, InvalidRegionContext, RegionNotKnown, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void unassociateRegionForUpdates(Region theRegion, int theObject) throws ObjectNotKnown, InvalidRegionContext, RegionNotKnown, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void subscribeObjectClassAttributesWithRegion(int theClass, Region theRegion, AttributeHandleSet attributeList) throws ObjectClassNotDefined, AttributeNotDefined, RegionNotKnown, InvalidRegionContext, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void subscribeObjectClassAttributesPassivelyWithRegion(int theClass, Region theRegion, AttributeHandleSet attributeList) throws ObjectClassNotDefined, AttributeNotDefined, RegionNotKnown, InvalidRegionContext, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void unsubscribeObjectClassWithRegion(int theClass, Region theRegion) throws ObjectClassNotDefined, RegionNotKnown, FederateNotSubscribed, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void subscribeInteractionClassWithRegion(int theClass, Region theRegion) throws InteractionClassNotDefined, RegionNotKnown, InvalidRegionContext, FederateLoggingServiceCalls, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void subscribeInteractionClassPassivelyWithRegion(int theClass, Region theRegion) throws InteractionClassNotDefined, RegionNotKnown, InvalidRegionContext, FederateLoggingServiceCalls, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void unsubscribeInteractionClassWithRegion(int theClass, Region theRegion) throws InteractionClassNotDefined, InteractionClassNotSubscribed, RegionNotKnown, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void sendInteractionWithRegion(int theInteraction, SuppliedParameters theParameters, byte[] userSuppliedTag, Region theRegion) throws InteractionClassNotDefined, InteractionClassNotPublished, InteractionParameterNotDefined, RegionNotKnown, InvalidRegionContext, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public EventRetractionHandle sendInteractionWithRegion(int theInteraction, SuppliedParameters theParameters, byte[] userSuppliedTag, Region theRegion, LogicalTime theTime) throws InteractionClassNotDefined, InteractionClassNotPublished, InteractionParameterNotDefined, InvalidFederationTime, RegionNotKnown, InvalidRegionContext, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void requestClassAttributeValueUpdateWithRegion(int theClass, AttributeHandleSet theAttributes, Region theRegion) throws ObjectClassNotDefined, AttributeNotDefined, RegionNotKnown, FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError, ConcurrentAccessAttempted {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getObjectClassHandle(String theName) throws NameNotFound, FederateNotExecutionMember, RTIinternalError {
        if (theName == null || theName.length() == 0) {
            throw new RTIinternalError("Incorrect or empty object class name");
        }

        CertiMessage request = new GetObjectClassHandle(federationTime, theName);

        try {
            CertiMessage response = processRequest(request);
            return (int) ((GetObjectClassHandle) response).getObjectClass(); //hack handles maju rozdielne velkosti
        } catch (NameNotFound ex) {
            throw ex;
        } catch (FederateNotExecutionMember ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {//TODO log

            System.out.println("Get object class handle");
        }

        return -1;
    }

    public String getObjectClassName(int theHandle) throws ObjectClassNotDefined, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getAttributeHandle(String theName, int whichClass) throws ObjectClassNotDefined, NameNotFound, FederateNotExecutionMember, RTIinternalError {
        CertiMessage request = new GetAttributeHandle(federationTime, theName, whichClass);
        try {
            CertiMessage response = processRequest(request);
            return ((GetAttributeHandle) response).getAttribute();
        } catch (ObjectClassNotDefined ex) {
            throw ex;
        } catch (NameNotFound ex) {
            throw ex;
        } catch (FederateNotExecutionMember ex) {
            throw ex;
        } catch (RTIinternalError ex) {
            throw ex;
        } catch (RuntimeException ex) {
            throw ex;
        } catch (Exception ex) {//TODO Log
        }

        return -1;
    }

    public String getAttributeName(int theHandle, int whichClass) throws ObjectClassNotDefined, AttributeNotDefined, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getInteractionClassHandle(String theName) throws NameNotFound, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public String getInteractionClassName(int theHandle) throws InteractionClassNotDefined, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getParameterHandle(String theName, int whichClass) throws InteractionClassNotDefined, NameNotFound, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public String getParameterName(int theHandle, int whichClass) throws InteractionClassNotDefined, InteractionParameterNotDefined, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getObjectInstanceHandle(String theName) throws ObjectNotKnown, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public String getObjectInstanceName(int theHandle) throws ObjectNotKnown, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getRoutingSpaceHandle(String theName) throws NameNotFound, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public String getRoutingSpaceName(int theHandle) throws SpaceNotDefined, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getDimensionHandle(String theName, int whichSpace) throws SpaceNotDefined, NameNotFound, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public String getDimensionName(int theHandle, int whichClass) throws SpaceNotDefined, DimensionNotDefined, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getAttributeRoutingSpaceHandle(int theHandle, int whichClass) throws ObjectClassNotDefined, AttributeNotDefined, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getObjectClass(int theObject) throws ObjectNotKnown, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getInteractionRoutingSpaceHandle(int theHandle) throws InteractionClassNotDefined, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getTransportationHandle(String theName) throws NameNotFound, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public String getTransportationName(int theHandle) throws InvalidTransportationHandle, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getOrderingHandle(String theName) throws NameNotFound, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public String getOrderingName(int theHandle) throws InvalidOrderingHandle, FederateNotExecutionMember, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void enableClassRelevanceAdvisorySwitch() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void disableClassRelevanceAdvisorySwitch() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void enableAttributeRelevanceAdvisorySwitch() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void disableAttributeRelevanceAdvisorySwitch() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void enableAttributeScopeAdvisorySwitch() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void disableAttributeScopeAdvisorySwitch() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void enableInteractionRelevanceAdvisorySwitch() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void disableInteractionRelevanceAdvisorySwitch() throws FederateNotExecutionMember, SaveInProgress, RestoreInProgress, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Region getRegion(int regionToken) throws FederateNotExecutionMember, ConcurrentAccessAttempted, RegionNotKnown, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getRegionToken(Region region) throws FederateNotExecutionMember, ConcurrentAccessAttempted, RegionNotKnown, RTIinternalError {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void tick() throws RTIinternalError, ConcurrentAccessAttempted {
        try {
            tickKernel(true, 0.0, Double.MAX_VALUE);
        } catch (SpecifiedSaveLabelDoesNotExist ex) {
            //TODO log
            throw new RTIinternalError("SpecifiedSaveLabelDoesNotExist in tick");
        }
    }

    private CertiMessage processRequest(CertiMessage request) throws ArrayIndexOutOfBounds, AsynchronousDeliveryAlreadyEnabled, AsynchronousDeliveryAlreadyDisabled, AttributeAlreadyOwned, AttributeAlreadyBeingAcquired, AttributeAlreadyBeingDivested, AttributeDivestitureWasNotRequested, AttributeAcquisitionWasNotRequested, AttributeNotDefined, AttributeNotKnown, AttributeNotOwned, AttributeNotPublished, RTIinternalError, ConcurrentAccessAttempted, CouldNotDiscover, CouldNotOpenFED, CouldNotRestore, DeletePrivilegeNotHeld, ErrorReadingFED, EventNotKnown, FederateAlreadyExecutionMember, FederateInternalError, FederateNotExecutionMember, FederateOwnsAttributes, FederatesCurrentlyJoined, FederateWasNotAskedToReleaseAttribute, FederationExecutionAlreadyExists, FederationExecutionDoesNotExist, FederationTimeAlreadyPassed, RegionNotKnown, InteractionClassNotDefined, InteractionClassNotKnown, InteractionClassNotPublished, InteractionParameterNotDefined, InteractionParameterNotKnown, InvalidExtents, InvalidFederationTime, InvalidLookahead, InvalidOrderingHandle, InvalidResignAction, InvalidRetractionHandle, InvalidTransportationHandle, NameNotFound, ObjectClassNotDefined, ObjectClassNotKnown, ObjectClassNotPublished, ObjectClassNotSubscribed, ObjectNotKnown, ObjectAlreadyRegistered, RestoreInProgress, RestoreNotRequested, SpaceNotDefined, SaveInProgress, SaveNotInitiated, SpecifiedSaveLabelDoesNotExist, TimeAdvanceAlreadyInProgress, TimeAdvanceWasNotInProgress, UnableToPerformSave, MemoryExhausted, ValueLengthExceeded, ValueCountExceeded {
        request.writeMessage(this.messageBuffer);

        try {
            System.out.println("Sending message (" + request.toString() + ")");
            this.messageBuffer.send(this.socket.getOutputStream());
        } catch (IOException ex) {
            System.err.println("libRTI: exception: NetworkError (write)");
            throw new RTIinternalError("libRTI: Network Write Error");
        }

        CertiMessage response = null;
        try {
            InputStream in = this.socket.getInputStream();

            response = this.messageBuffer.receive(in);
            System.out.println("Received: " + response.toString() + "\n");

            if (request.getType() != (response.getType())) {
                throw new RTIinternalError("RTI Ambassador Process request: request type != response type");
            }
        } catch (IOException ex) {
            System.err.println("libRTI: exception: NetworkError (read)");
            throw new RTIinternalError("libRTI: Network Read Error");
        } catch (CertiException ex) {
            translateException(ex);
        }

        return response;
    }

    private void translateException(CertiException ex) throws ArrayIndexOutOfBounds, AsynchronousDeliveryAlreadyEnabled, AsynchronousDeliveryAlreadyDisabled, AttributeAlreadyOwned, AttributeAlreadyBeingAcquired, AttributeAlreadyBeingDivested, AttributeDivestitureWasNotRequested, AttributeAcquisitionWasNotRequested, AttributeNotDefined, AttributeNotKnown, AttributeNotOwned, AttributeNotPublished, RTIinternalError, ConcurrentAccessAttempted, CouldNotDiscover, CouldNotOpenFED, CouldNotRestore, DeletePrivilegeNotHeld, ErrorReadingFED, EventNotKnown, FederateAlreadyExecutionMember, FederateInternalError, FederateNotExecutionMember, FederateOwnsAttributes, FederatesCurrentlyJoined, FederateWasNotAskedToReleaseAttribute, FederationExecutionAlreadyExists, FederationExecutionDoesNotExist, FederationTimeAlreadyPassed, RegionNotKnown, InteractionClassNotDefined, InteractionClassNotKnown, InteractionClassNotPublished, InteractionParameterNotDefined, InteractionParameterNotKnown, InvalidExtents, InvalidFederationTime, InvalidLookahead, InvalidOrderingHandle, InvalidResignAction, InvalidRetractionHandle, InvalidTransportationHandle, NameNotFound, ObjectClassNotDefined, ObjectClassNotKnown, ObjectClassNotPublished, ObjectClassNotSubscribed, ObjectNotKnown, ObjectAlreadyRegistered, RestoreInProgress, RestoreNotRequested, SpaceNotDefined, SaveInProgress, SaveNotInitiated, SpecifiedSaveLabelDoesNotExist, TimeAdvanceAlreadyInProgress, TimeAdvanceWasNotInProgress, UnableToPerformSave, MemoryExhausted, ValueLengthExceeded, ValueCountExceeded {
        switch (ex.getExceptionType()) {
            case NO_EXCEPTION:
                break;
            case ArrayIndexOutOfBounds: {
                System.out.println("Throwing ArrayIndexOutOfBounds exception.");
                throw new ArrayIndexOutOfBounds(ex.getReason());
            }
            case AsynchronousDeliveryAlreadyEnabled: {
                System.out.println("Throwing AsynchronousDeliveryAlreadyEnabled exception.");
                throw new AsynchronousDeliveryAlreadyEnabled(ex.getReason());
            }
            case AsynchronousDeliveryAlreadyDisabled: {
                System.out.println("Throwing AsynchronousDeliveryAlreadyDisabled exception.");
                throw new AsynchronousDeliveryAlreadyDisabled(ex.getReason());
            }
            case AttributeAlreadyOwned: {
                System.out.println("Throwing AttributeAlreadyOwned exception.");
                throw new AttributeAlreadyOwned(ex.getReason());
            }
            case AttributeAlreadyBeingAcquired: {
                System.out.println("Throwing AttributeAlreadyBeingAcquired exception.");
                throw new AttributeAlreadyBeingAcquired(ex.getReason());
            }
            case AttributeAlreadyBeingDivested: {
                System.out.println("Throwing AttributeAlreadyBeingDivested exception.");
                throw new AttributeAlreadyBeingDivested(ex.getReason());
            }
            case AttributeDivestitureWasNotRequested: {
                System.out.println("Throwing AttributeDivestitureWasNotRequested exception.");
                throw new AttributeDivestitureWasNotRequested(ex.getReason());
            }
            case AttributeAcquisitionWasNotRequested: {
                System.out.println("Throwing AttributeAcquisitionWasNotRequested exception.");
                throw new AttributeAcquisitionWasNotRequested(ex.getReason());
            }
            case AttributeNotDefined: {
                System.out.println("Throwing AttributeNotDefined exception.");
                throw new AttributeNotDefined(ex.getReason());
            }
            case AttributeNotKnown: {
                System.out.println("Throwing AttributeNotKnown exception.");
                throw new AttributeNotKnown(ex.getReason());
            }
            case AttributeNotOwned: {
                System.out.println("Throwing AttributeNotOwned exception.");
                throw new AttributeNotOwned(ex.getReason());
            }
            case AttributeNotPublished: {
                System.out.println("Throwing AttributeNotPublished exception.");
                throw new AttributeNotPublished(ex.getReason());
            }
            case AttributeNotSubscribed: {
                System.out.println("Throwing AttributeNotSubscribed exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case ConcurrentAccessAttempted: {
                System.out.println("Throwing ConcurrentAccessAttempted exception.");
                throw new ConcurrentAccessAttempted(ex.getReason());
            }
            case CouldNotDiscover: {
                System.out.println("Throwing CouldNotDiscover exception.");
                throw new CouldNotDiscover(ex.getReason());
            }
            case CouldNotOpenFED: {
                System.out.println("Throwing CouldNotOpenFED exception.");
                throw new CouldNotOpenFED(ex.getReason());
            }
            case CouldNotOpenRID: {
                System.out.println("Throwing CouldNotOpenRID exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case CouldNotRestore: {
                System.out.println("Throwing CouldNotRestore exception.");
                throw new CouldNotRestore(ex.getReason());
            }
            case DeletePrivilegeNotHeld: {
                System.out.println("Throwing DeletePrivilegeNotHeld exception.");
                throw new DeletePrivilegeNotHeld(ex.getReason());
            }
            case ErrorReadingRID: {
                System.out.println("Throwing ErrorReadingRID exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case ErrorReadingFED: {
                System.out.println("Throwing ErrorReadingFED exception.");
                throw new ErrorReadingFED(ex.getReason());
            }
            case EventNotKnown: {
                System.out.println("Throwing EventNotKnown exception.");
                throw new EventNotKnown(ex.getReason());
            }
            case FederateAlreadyPaused: {
                System.out.println("Throwing FederateAlreadyPaused exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case FederateAlreadyExecutionMember: {
                System.out.println(
                        "Throwing FederateAlreadyExecutionMember exception.");
                throw new FederateAlreadyExecutionMember(ex.getReason());
            }
            case FederateDoesNotExist: {
                System.out.println("Throwing FederateDoesNotExist exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case FederateInternalError: {
                System.out.println("Throwing FederateInternalError exception.");
                throw new FederateInternalError(ex.getReason());
            }
            case FederateNameAlreadyInUse: {
                System.out.println("Throwing FederateNameAlreadyInUse exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case FederateNotExecutionMember: {
                System.out.println("Throwing FederateNotExecutionMember exception.");
                throw new FederateNotExecutionMember(ex.getReason());
            }
            case FederateNotPaused: {
                System.out.println("Throwing FederateNotPaused exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case FederateOwnsAttributes: {
                System.out.println("Throwing FederateOwnsAttributes exception.");
                throw new FederateOwnsAttributes(ex.getReason());
            }
            case FederatesCurrentlyJoined: {
                System.out.println("Throwing FederatesCurrentlyJoined exception.");
                throw new FederatesCurrentlyJoined(ex.getReason());
            }
            case FederateWasNotAskedToReleaseAttribute: {
                System.out.println("Throwing FederateWasNotAskedToReleaseAttribute exception.");
                throw new FederateWasNotAskedToReleaseAttribute(ex.getReason());
            }
            case FederationAlreadyPaused: {
                System.out.println("Throwing FederationAlreadyPaused exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case FederationExecutionAlreadyExists:
                System.out.println("Throwing FederationExecutionAlreadyExists excep.");
                throw new FederationExecutionAlreadyExists(ex.getReason());
            case FederationExecutionDoesNotExist: {
                System.out.println("Throwing FederationExecutionDoesNotExist except.");
                throw new FederationExecutionDoesNotExist(ex.getReason());
            }
            case FederationNotPaused: {
                System.out.println("Throwing FederationNotPaused exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case FederationTimeAlreadyPassed: {
                System.out.println("Throwing FederationTimeAlreadyPassed exception.");
                throw new FederationTimeAlreadyPassed(ex.getReason());
            }
            case FederateNotPublishing: {
                System.out.println("Throwing FederateNotPublishing exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case FederateNotSubscribing: {
                System.out.println("Throwing FederateNotSubscribing exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case RegionNotKnown: {
                System.out.println("Throwing RegionNotKnown exception.");
                throw new RegionNotKnown(ex.getReason());
            }
            case IDsupplyExhausted: {
                System.out.println("Throwing IDsupplyExhausted exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case InteractionClassNotDefined: {
                System.out.println("Throwing InteractionClassNotDefined exception.");
                throw new InteractionClassNotDefined(ex.getReason());
            }
            case InteractionClassNotKnown: {
                System.out.println("Throwing InteractionClassNotKnown exception.");
                throw new InteractionClassNotKnown(ex.getReason());
            }
            case InteractionClassNotPublished: {
                System.out.println("Throwing InteractionClassNotPublished exception.");
                throw new InteractionClassNotPublished(ex.getReason());
            }
            case InteractionParameterNotDefined: {
                System.out.println(
                        "Throwing InteractionParameterNotDefined exception.");
                throw new InteractionParameterNotDefined(ex.getReason());
            }
            case InteractionParameterNotKnown: {
                System.out.println("Throwing InteractionParameterNotKnown exception.");
                throw new InteractionParameterNotKnown(ex.getReason());
            }
            case InvalidDivestitureCondition: {
                System.out.println("Throwing InvalidDivestitureCondition exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case InvalidExtents: {
                System.out.println("Throwing InvalidExtents exception.");
                throw new InvalidExtents(ex.getReason());
            }
            case InvalidFederationTime: {
                System.out.println("Throwing InvalidFederationTime exception.");
                throw new InvalidFederationTime(ex.getReason());
            }
            case InvalidFederationTimeDelta: {
                System.out.println("Throwing InvalidFederationTimeDelta exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case InvalidLookahead: {
                System.out.println("Throwing InvalidLookahead.");
                throw new InvalidLookahead(ex.getReason());
            }
            case InvalidObjectHandle: {
                System.out.println("Throwing InvalidObjectHandle exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case InvalidOrderingHandle: {
                System.out.println("Throwing InvalidOrderingHandle exception.");
                throw new InvalidOrderingHandle(ex.getReason());
            }
            case InvalidResignAction: {
                System.out.println("Throwing InvalidResignAction exception.");
                throw new InvalidResignAction(ex.getReason());
            }
            case InvalidRetractionHandle: {
                System.out.println("Throwing InvalidRetractionHandle exception.");
                throw new InvalidRetractionHandle(ex.getReason());
            }
            case InvalidRoutingSpace: {
                System.out.println("Throwing InvalidRoutingSpace exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case InvalidTransportationHandle: {
                System.out.println("Throwing InvalidTransportationHandle exception.");
                throw new InvalidTransportationHandle(ex.getReason());
            }
            case MemoryExhausted: {
                System.out.println("Throwing MemoryExhausted exception.");
                throw new MemoryExhausted(ex.getReason());
            }
            case NameNotFound: {
                System.out.println("Throwing NameNotFound exception.");
                throw new NameNotFound(ex.getReason());
            }
            case NoPauseRequested: {
                System.out.println("Throwing NoPauseRequested exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case NoResumeRequested: {
                System.out.println("Throwing NoResumeRequested exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case ObjectClassNotDefined: {
                System.out.println("Throwing ObjectClassNotDefined exception.");
                throw new ObjectClassNotDefined(ex.getReason());
            }
            case ObjectClassNotKnown: {
                System.out.println("Throwing ObjectClassNotKnown exception.");
                throw new ObjectClassNotKnown(ex.getReason());
            }
            case ObjectClassNotPublished: {
                System.out.println("Throwing ObjectClassNotPublished exception.");
                throw new ObjectClassNotPublished(ex.getReason());
            }
            case ObjectClassNotSubscribed: {
                System.out.println("Throwing ObjectClassNotSubscribed exception.");
                throw new ObjectClassNotSubscribed(ex.getReason());
            }
            case ObjectNotKnown: {
                System.out.println("Throwing ObjectNotKnown exception.");
                throw new ObjectNotKnown(ex.getReason());
            }
            case ObjectAlreadyRegistered: {
                System.out.println("Throwing ObjectAlreadyRegistered exception.");
                throw new ObjectAlreadyRegistered(ex.getReason());
            }
            case RestoreInProgress: {
                System.out.println("Throwing RestoreInProgress exception.");
                throw new RestoreInProgress(ex.getReason());
            }
            case RestoreNotRequested: {
                System.out.println("Throwing RestoreNotRequested exception.");
                throw new RestoreNotRequested(ex.getReason());
            }
            case RTIinternalError: {
                System.out.println("Throwing RTIinternalError exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case SpaceNotDefined: {
                System.out.println("Throwing SpaceNotDefined exception.");
                throw new SpaceNotDefined(ex.getReason());
            }
            case SaveInProgress: {
                System.out.println("Throwing SaveInProgress exception.");
                throw new SaveInProgress(ex.getReason());
            }
            case SaveNotInitiated: {
                System.out.println("Throwing SaveNotInitiated exception.");
                throw new SaveNotInitiated(ex.getReason());
            }
            case SecurityError: {
                System.out.println("Throwing SecurityError exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case SpecifiedSaveLabelDoesNotExist: {
                System.out.println("Throwing SpecifiedSaveLabelDoesNotExist exception.");
                throw new SpecifiedSaveLabelDoesNotExist(ex.getReason());
            }
            case TimeAdvanceAlreadyInProgress: {
                System.out.println("Throwing TimeAdvanceAlreadyInProgress exception.");
                throw new TimeAdvanceAlreadyInProgress(ex.getReason());
            }
            case TimeAdvanceWasNotInProgress: {
                System.out.println("Throwing TimeAdvanceWasNotInProgress exception.");
                throw new TimeAdvanceWasNotInProgress(ex.getReason());
            }
            case TooManyIDsRequested: {
                System.out.println("Throwing TooManyIDsRequested exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case UnableToPerformSave: {
                System.out.println("Throwing UnableToPerformSave exception.");
                throw new UnableToPerformSave(ex.getReason());
            }
            case UnimplementedService: {
                System.out.println("Throwing UnimplementedService exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case UnknownLabel: {
                System.out.println("Throwing UnknownLabel exception.");
                throw new RTIinternalError(ex.getReason());
            }
            case ValueCountExceeded: {
                System.out.println("Throwing ValueCountExceeded exception.");
                throw new ValueCountExceeded(ex.getReason());
            }
            case ValueLengthExceeded: {
                System.out.println("Throwing ValueLengthExceeded exception.");
                throw new ValueLengthExceeded(ex.getReason());
            }
            default: {
                System.out.println("Throwing unknown exception !");
                throw new RTIinternalError(ex.getReason());
            }
        }
    }

    private void callFederateAmbassador(CertiMessage message) throws RTIinternalError {
        //TODO implement other messages
        try {
            switch (message.getType()) {
                case PROVIDE_ATTRIBUTE_VALUE_UPDATE:
                    federateAmbassador.provideAttributeValueUpdate((int) ((ProvideAttributeValueUpdate) message).getObjectClass(), ((ProvideAttributeValueUpdate) message).getAttributeHandleSet());
                    break;
                case START_REGISTRATION_FOR_OBJECT_CLASS:
                    federateAmbassador.startRegistrationForObjectClass((int) ((StartRegistrationForObjectClass) message).getObjectClass());
                    break;
                default:
                    throw new RTIinternalError("RTI service requested by RTI is unknown.");
            }
        } catch (Exception ex) {
            //TODO log
            System.out.println("Error in " + message.getType() + ": " + ex.getMessage());
            throw new RTIinternalError(ex.getMessage());
        }
    }
}
