// ----------------------------------------------------------------------------
// CERTI - HLA Run Time Infrastructure
// Copyright (C) 2010 Andrej Pancik
//
// This program is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// ----------------------------------------------------------------------------
package certi.rti.impl;

import hla.rti.ArrayIndexOutOfBounds;
import hla.rti.SuppliedParameters;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class CertiSuppliedParameters extends HashMap<Integer, byte[]> implements SuppliedParameters {

    private List<Integer> handles = new ArrayList<Integer>();

    public CertiSuppliedParameters() {
        super();
    }

    public CertiSuppliedParameters(int size) {
        super(size);
        handles = new ArrayList<Integer>(size);
    }

    public void add(int handle, byte[] value) {
        if (put(handle, value) == null) {
            handles.add(handle);
        }
    }

    public void empty() {
        clear();
        handles.clear();
    }

    public int getHandle(int index) throws ArrayIndexOutOfBounds {
        try {
            return handles.get(index);
        } catch (IndexOutOfBoundsException exception) {
            throw new ArrayIndexOutOfBounds(exception.getMessage());
        }
    }

    public byte[] getValue(int index) throws ArrayIndexOutOfBounds {
        return getValueReference(index).clone();
    }

    public int getValueLength(int index) throws ArrayIndexOutOfBounds {
        return getValueReference(index).length;
    }

    public byte[] getValueReference(int index) throws ArrayIndexOutOfBounds {
        try {
            return get(handles.get(index));
        } catch (IndexOutOfBoundsException exception) {
            throw new ArrayIndexOutOfBounds(exception.getMessage());
        }
    }

    public void remove(int handle) throws ArrayIndexOutOfBounds {
        remove(handles.remove(handle));
    }

    public void removeAt(int index) throws ArrayIndexOutOfBounds {
        try {
            remove(handles.remove(index));
        } catch (IndexOutOfBoundsException exception) {
            throw new ArrayIndexOutOfBounds(exception.getMessage());
        }
    }
}
